@@ -28,6 +28,7 @@
 
 #include "base_wx/platform.h"
 #include "base_wx/huginConfig.h"
+#include "base_wx/LensTools.h"
 
 #include "PT/ImageGraph.h"
 #include "base_wx/wxPlatform.h"
@@ -39,10 +40,13 @@
 #include "hugin/MainFrame.h"
 #include "hugin/huginApp.h"
 #include "hugin/TextKillFocusHandler.h"
-#include "hugin/HFOVDialog.h"
 #include "hugin/config_defaults.h"
 #include "hugin/wxPanoCommand.h"
 
+/*#ifdef __WXMAC__
+	#include <Foundation/Foundation.h>
+#endif */
+
 using namespace hugin_utils;
 using namespace std;
 
@@ -457,7 +461,52 @@ void AssistantPanel::OnAlignSendToBatch(wxCommandEvent &e)
 	if(wxFileName::FileExists(projectFile))
 	{
 #if defined __WXMAC__ && defined MAC_SELF_CONTAINED_BUNDLE
-        wxExecute(_T("open -b net.sourceforge.hugin.PTBatcherGUI -a"+wxQuoteFilename(projectFile)));
+		// Original patch for OSX by Charlie Reiman dd. 18 June 2011
+		// Slightly modified by HvdW. Errors in here are mine, not Charlie's. 
+		FSRef appRef;
+		FSRef actuallyLaunched;
+		OSStatus err;
+		FSRef documentArray[1]; // Don't really need an array if we only have 1 item
+		LSLaunchFSRefSpec launchSpec;
+		Boolean  isDir;
+		
+		err = LSFindApplicationForInfo(kLSUnknownCreator,
+									   CFSTR("net.sourceforge.hugin.PTBatcherGUI"),
+									   NULL,
+									   &appRef,
+									   NULL);
+		if (err != noErr) {
+			// error, can't find PTBatcherGUI
+			wxMessageBox(wxString::Format(_("External program %s not found in the bundle, reverting to system path"), wxT("open")), _("Error"));
+			// Possibly a silly attempt otherwise the previous would have worked as well, but just try it.
+			wxExecute(_T("open -b net.sourceforge.hugin.PTBatcherGUI ")+wxQuoteFilename(projectFile));
+		}
+		
+		wxCharBuffer projectFilebuffer=projectFile.ToUTF8();
+		// Point to document
+		err = FSPathMakeRef((unsigned char*) projectFilebuffer.data(), &documentArray[0], &isDir);
+		if (err != noErr || isDir) {
+			// Something went wrong.
+			wxMessageBox(wxString::Format(_("Project file not found"), wxT("open")), _("Error"));
+		}
+		launchSpec.appRef = &appRef;
+		launchSpec.numDocs = sizeof(documentArray)/sizeof(documentArray[0]);
+		launchSpec.itemRefs = documentArray;
+		launchSpec.passThruParams = NULL;
+		launchSpec.launchFlags = kLSLaunchDontAddToRecents + kLSLaunchDontSwitch;
+		launchSpec.asyncRefCon = NULL;
+		
+		err = LSOpenFromRefSpec(&launchSpec, &actuallyLaunched);
+		if (err != noErr && err != kLSLaunchInProgressErr) {  // Should be ok if it's in progress... I think. 
+			// Launch failed.
+			wxMessageBox(wxString::Format(_("Can't launch PTBatcherGui"), wxT("open")), _("Error"));
+		}
+		
+		// Should verify that actuallyLaunched and appRef are the same.
+		if (FSCompareFSRefs(&appRef, &actuallyLaunched) != noErr) {
+			// error, lauched the wrong thing.
+			wxMessageBox(wxString::Format(_("Launched incorrect programme"), wxT("open")), _("Error"));
+		}
 #else
 #ifdef __WINDOWS__
 		wxString huginPath = getExePath(wxGetApp().argv[0])+wxFileName::GetPathSeparator(); 
@@ -497,43 +546,20 @@ void AssistantPanel::OnLoadLens(wxCommandEvent & e)
 {
     unsigned int imgNr = 0;
     unsigned int lensNr = m_variable_groups->getLenses().getPartNumber(imgNr);
-    Lens lens = m_variable_groups->getLensForImage(imgNr);
-    VariableMap vars = m_pano->getImageVariables(imgNr);
-    ImageOptions imgopts = m_pano->getImage(imgNr).getOptions();
-
-    if (LoadLensParametersChoose(this, lens, vars, imgopts)) {
-        /** @todo maybe this isn't the best way to load the lens data.
-         * Check with LoadLensParamtersChoose how this is done, and use only
-         * the image variables, rather than merging in the lens ones.
-         */
-        for (LensVarMap::iterator it = lens.variables.begin();
-             it != lens.variables.end(); it++)
+    // get all images with the current lens.
+    UIntSet imgs;
+    for (unsigned int i = 0; i < m_pano->getNrOfImages(); i++)
+    {
+        if (m_variable_groups->getLenses().getPartNumber(i) == lensNr)
         {
-            vars.insert(pair<std::string, HuginBase::Variable>(
-                        it->first,
-                        HuginBase::Variable(it->second.getName(),
-                                            it->second.getValue() )
-                    ));
-        }
-        /** @todo I think the sensor size should be copied over,
-         * but SrcPanoImage doesn't have such a variable yet.
-         */
-        std::vector<PanoCommand*> commands;
-        commands.push_back(new PT::UpdateImageVariablesCmd(*m_pano, imgNr, vars));
-        // get all images with the current lens.
-        UIntSet imgs;
-        for (unsigned int i = 0; i < m_pano->getNrOfImages(); i++) {
-            if (m_variable_groups->getLenses().getPartNumber(i) == lensNr) {
-                imgs.insert(i);
-            }
-        }
-
-        // set image options.
-        commands.push_back(new PT::SetImageOptionsCmd(*m_pano, imgopts, imgs) );
-        GlobalCmdHist::getInstance().addCommand(
-                new PT::CombinedPanoCommand(*m_pano, commands));
+            imgs.insert(i);
+        };
     }
-
+    PT::PanoCommand* cmd=NULL;
+    if(ApplyLensParameters(this,m_pano,imgs,cmd))
+    {
+        GlobalCmdHist::getInstance().addCommand(cmd);
+    };
 }
 
 void AssistantPanel::OnExifToggle (wxCommandEvent & e)
@@ -551,7 +577,6 @@ void AssistantPanel::OnExifToggle (wxCommandEvent & e)
                 srcImg.setHFOV(50);
             }
         }
-                //initLensFromFile(pano.getImage(imgNr).getFilename().c_str(), c, lens, vars, imgopts, true);
         GlobalCmdHist::getInstance().addCommand(
                 new PT::UpdateSrcImageCmd( *m_pano, imgNr, srcImg)
                                                );
