@@ -114,7 +114,6 @@ enum{
 
 BEGIN_EVENT_TABLE(GLwxAuiFloatingFrame, wxAuiFloatingFrame)
     EVT_ACTIVATE(GLwxAuiFloatingFrame::OnActivate)
-//    EVT_CLOSE(GLwxAuiFloatingFrame::OnClose)
 END_EVENT_TABLE()
 
 BEGIN_EVENT_TABLE(GLPreviewFrame, wxFrame)
@@ -165,6 +164,7 @@ BEGIN_EVENT_TABLE(GLPreviewFrame, wxFrame)
     EVT_TEXT_ENTER(XRCID("pano_val_roi_top"), GLPreviewFrame::OnROIChanged)
     EVT_TEXT_ENTER(XRCID("pano_val_roi_right"), GLPreviewFrame::OnROIChanged)
     EVT_TEXT_ENTER(XRCID("pano_val_roi_bottom"), GLPreviewFrame::OnROIChanged)
+    EVT_BUTTON(XRCID("reset_crop_button"), GLPreviewFrame::OnResetCrop)
     EVT_TEXT_ENTER(XRCID("exposure_text"), GLPreviewFrame::OnExposureChanged)
     EVT_COMMAND_RANGE(PROJ_PARAM_VAL_ID,PROJ_PARAM_VAL_ID+PANO_PROJECTION_MAX_PARMS,wxEVT_COMMAND_TEXT_ENTER,GLPreviewFrame::OnProjParameterChanged)
     EVT_BUTTON(PROJ_PARAM_RESET_ID, GLPreviewFrame::OnProjParameterReset)
@@ -777,7 +777,7 @@ GLPreviewFrame::~GLPreviewFrame()
 
 void GLPreviewFrame::InitPreviews()
 {
-    if(!preview_helper || !panosphere_overview_helper || !plane_overview_helper)  
+    if(preview_helper==NULL || panosphere_overview_helper==NULL || plane_overview_helper==NULL)
     {
         m_GLPreview->SetUpContext();
         m_GLOverview->SetUpContext();
@@ -785,17 +785,6 @@ void GLPreviewFrame::InitPreviews()
     };
 };
 
-bool GLwxAuiManager::ProcessDockResult(wxAuiPaneInfo& target,
-                                   const wxAuiPaneInfo& new_pos)
-{
-//    std::cout << "target: " << std::bitset<std::numeric_limits<unsigned int>::digits>(target.state) << std::endl;
-//    std::cout << "target: " << target.dock_direction << " " << target.dock_layer << " " << target.dock_row << " " << target.dock_pos << " " << target.state << std::endl;
-//    std::cout << "newpos: " << std::bitset<std::numeric_limits<unsigned int>::digits>(new_pos.state) << std::endl;
-//    std::cout << "newpos: " << new_pos.dock_direction << " " << new_pos.dock_layer << " " << new_pos.dock_row << " " << new_pos.dock_pos << " " << new_pos.state << std::endl;
-    return wxAuiManager::ProcessDockResult(target, new_pos);
-}
-
-
 /**
 * Update tools and GUI elements according to blend mode choice
 */
@@ -2410,6 +2399,13 @@ void GLPreviewFrame::OnROIChanged ( wxCommandEvent & e )
                                            );
 };
 
+void GLPreviewFrame::OnResetCrop(wxCommandEvent &e)
+{
+    PanoramaOptions opt=m_pano.getOptions();
+    opt.setROI(vigra::Rect2D(0,0,opt.getWidth(),opt.getHeight()));
+    GlobalCmdHist::getInstance().addCommand(new PT::SetPanoOptionsCmd(m_pano,opt));
+};
+
 void GLPreviewFrame::OnHFOVChanged ( wxCommandEvent & e )
 {
     PanoramaOptions opt = m_pano.getOptions();
