@@ -29,6 +29,7 @@
 #include <exiv2/exif.hpp>
 #include <exiv2/image.hpp>
 
+#include <wx/dir.h>
 #include "panoinc_WX.h"
 #include "panoinc.h"
 
@@ -69,6 +70,10 @@
 
 #include "hugin/AboutDialog.h"
 
+#if HUGIN_HSI
+#include "PluginItems.h"
+#endif
+
 using namespace PT;
 using namespace std;
 using namespace hugin_utils;
@@ -157,8 +162,7 @@ BEGIN_EVENT_TABLE(MainFrame, wxFrame)
     EVT_MENU(XRCID("action_save_project"),  MainFrame::OnSaveProject)
     EVT_MENU(XRCID("action_save_as_project"),  MainFrame::OnSaveProjectAs)
     EVT_MENU(XRCID("action_save_as_ptstitcher"),  MainFrame::OnSavePTStitcherAs)
-	EVT_MENU(XRCID("action_send_to_batch"),  MainFrame::OnSendToBatch)
-	EVT_MENU(XRCID("action_open_batch_processor"),  MainFrame::OnOpenPTBatcher)
+    EVT_MENU(XRCID("action_open_batch_processor"),  MainFrame::OnOpenPTBatcher)
     EVT_MENU(XRCID("action_import_project"), MainFrame::OnMergeProject)
     EVT_MENU(XRCID("action_apply_template"),  MainFrame::OnApplyTemplate)
     EVT_MENU(XRCID("action_exit_hugin"),  MainFrame::OnUserQuit)
@@ -170,6 +174,9 @@ BEGIN_EVENT_TABLE(MainFrame, wxFrame)
     EVT_MENU(XRCID("action_show_faq"),  MainFrame::OnFAQ)
     EVT_MENU(XRCID("action_show_donate"),  MainFrame::OnShowDonate)
     EVT_MENU(XRCID("action_show_prefs"), MainFrame::OnShowPrefs)
+#ifdef HUGIN_HSI
+    EVT_MENU(XRCID("action_python_script"), MainFrame::OnPythonScript)
+#endif
     EVT_MENU(XRCID("ID_EDITUNDO"), MainFrame::OnUndo)
     EVT_MENU(XRCID("ID_EDITREDO"), MainFrame::OnRedo)
     EVT_MENU(XRCID("ID_SHOW_FULL_SCREEN"), MainFrame::OnFullScreen)
@@ -196,16 +203,12 @@ BEGIN_EVENT_TABLE(MainFrame, wxFrame)
     EVT_MENU(XRCID("ID_SHOW_PANEL_OPTIMIZER"), MainFrame::OnShowPanel)
     EVT_MENU(XRCID("ID_SHOW_PANEL_OPTIMIZER_PHOTOMETRIC"), MainFrame::OnShowPanel)
     EVT_MENU(XRCID("ID_SHOW_PANEL_PANORAMA"), MainFrame::OnShowPanel)
-    
-
-
-
     EVT_MENU(XRCID("action_add_images"),  MainFrame::OnAddImages)
     EVT_BUTTON(XRCID("action_add_images"),  MainFrame::OnAddImages)
     EVT_MENU(XRCID("action_add_time_images"),  MainFrame::OnAddTimeImages)
     EVT_BUTTON(XRCID("action_add_time_images"),  MainFrame::OnAddTimeImages)
     //EVT_NOTEBOOK_PAGE_CHANGED(XRCID( "controls_notebook"), MainFrame::UpdatePanels)
-	EVT_CLOSE(  MainFrame::OnExit)
+    EVT_CLOSE(  MainFrame::OnExit)
     EVT_SIZE(MainFrame::OnSize)
 END_EVENT_TABLE()
 
@@ -279,6 +282,56 @@ MainFrame::MainFrame(wxWindow* parent, Panorama & pano)
 #endif
     SetMenuBar(wxXmlResource::Get()->LoadMenuBar(this, wxT("main_menubar")));
 
+#ifdef HUGIN_HSI
+    wxMenuBar* menubar=GetMenuBar();
+    // the plugin menu will be generated dynamically
+    wxMenu *pluginMenu=new wxMenu();
+    // search for all .py files in plugins directory
+    wxDir dir(GetDataPath()+wxT("plugins"));
+    wxString filename;
+    bool cont=dir.GetFirst(&filename,wxT("*.py"),wxDIR_FILES|wxDIR_HIDDEN);
+    std::list<PluginItem> items;
+    while(cont)
+    {
+        wxFileName file(dir.GetName(),filename);
+        file.MakeAbsolute();
+        PluginItem item(file);
+        if(item.IsAPIValid())
+        {
+            items.push_back(item);
+        };
+        cont=dir.GetNext(&filename);
+    };
+    items.sort(comparePluginItem);
+
+    int pluginID=wxID_HIGHEST+2000;
+    for(list<PluginItem>::const_iterator it=items.begin();it!=items.end();it++)
+    {
+        PluginItem item=*it;
+        // skip incompatible plugins
+        int categoryID=pluginMenu->FindItem(item.GetCategory());
+        wxMenu* categoryMenu;
+        if(categoryID==wxNOT_FOUND)
+        {
+            categoryMenu=new wxMenu();
+            pluginMenu->AppendSubMenu(categoryMenu,item.GetCategory());
+        }
+        else
+        {
+            categoryMenu=pluginMenu->FindItem(categoryID)->GetSubMenu();
+        };
+        categoryMenu->Append(pluginID,item.GetName(),item.GetDescription());
+        m_plugins[pluginID]=item.GetFilename();
+        Connect(pluginID, wxEVT_COMMAND_MENU_SELECTED,wxCommandEventHandler(MainFrame::OnPlugin));
+        pluginID++;
+    };
+    // show the new menu
+    menubar->Insert(menubar->GetMenuCount()-2,pluginMenu,_("&Actions"));
+
+#else
+    GetMenuBar()->Enable(XRCID("action_python_script"), false);
+#endif
+
     // create tool bar
     SetToolBar(wxXmlResource::Get()->LoadToolBar(this, wxT("main_toolbar")));
 
@@ -406,7 +459,20 @@ MainFrame::MainFrame(wxWindow* parent, Panorama & pano)
 
     // set progress display for image cache.
     ImageCache::getInstance().setProgressDisplay(this);
+#if defined __WXMSW__
+    unsigned long long mem = HUGIN_IMGCACHE_UPPERBOUND;
+    unsigned long mem_low = wxConfigBase::Get()->Read(wxT("/ImageCache/UpperBound"), HUGIN_IMGCACHE_UPPERBOUND);
+    unsigned long mem_high = wxConfigBase::Get()->Read(wxT("/ImageCache/UpperBoundHigh"), (long) 0);
+    if (mem_high > 0) {
+      mem = ((unsigned long long) mem_high << 32) + mem_low;
+    }
+    else {
+      mem = mem_low;
+    }
+    ImageCache::getInstance().SetUpperLimit(mem);
+#else
     ImageCache::getInstance().SetUpperLimit(wxConfigBase::Get()->Read(wxT("/ImageCache/UpperBound"), HUGIN_IMGCACHE_UPPERBOUND));
+#endif
 
     if(splash) {
         splash->Close();
@@ -544,7 +610,7 @@ bool MainFrame::CloseProject(bool cancelable)
             {
                 wxCommandEvent dummy;
                 OnSaveProject(dummy);
-                return true;
+                return !pano.isDirty();
             }
             case wxID_CANCEL:
                 return false;
@@ -1290,7 +1356,20 @@ void MainFrame::OnShowPrefs(wxCommandEvent & e)
     pref_dlg->ShowModal();
     //update image cache size
     wxConfigBase* cfg=wxConfigBase::Get();
+#if defined __WXMSW__
+    unsigned long long mem = HUGIN_IMGCACHE_UPPERBOUND;
+    unsigned long mem_low = cfg->Read(wxT("/ImageCache/UpperBound"), HUGIN_IMGCACHE_UPPERBOUND);
+    unsigned long mem_high = cfg->Read(wxT("/ImageCache/UpperBoundHigh"), (long) 0);
+    if (mem_high > 0) {
+      mem = ((unsigned long long) mem_high << 32) + mem_low;
+    }
+    else {
+      mem = mem_low;
+    }
+    ImageCache::getInstance().SetUpperLimit(mem);
+#else
     ImageCache::getInstance().SetUpperLimit(cfg->Read(wxT("/ImageCache/UpperBound"), HUGIN_IMGCACHE_UPPERBOUND));
+#endif
     images_panel->ReloadCPDetectorSettings();
     gl_preview_frame->SetShowProjectionHints(cfg->Read(wxT("/GLPreviewFrame/ShowProjectionHints"),HUGIN_SHOW_PROJECTION_HINTS)!=0);
 
@@ -1369,7 +1448,8 @@ void MainFrame::OnOptimize(wxCommandEvent & e)
 void MainFrame::OnDoStitch(wxCommandEvent & e)
 {
     DEBUG_TRACE("");
-    pano_panel->DoStitch();
+    wxCommandEvent cmdEvt(wxEVT_COMMAND_BUTTON_CLICKED,XRCID("pano_button_stitch"));
+    pano_panel->GetEventHandler()->AddPendingEvent(cmdEvt);
 }
 
 void MainFrame::OnMergeProject(wxCommandEvent & e)
@@ -1443,12 +1523,6 @@ void MainFrame::OnApplyTemplate(wxCommandEvent & e)
     }
 }
 
-
-void MainFrame::OnSendToBatch(wxCommandEvent & e)
-{
-	pano_panel->SendToBatch();
-}
-
 void MainFrame::OnOpenPTBatcher(wxCommandEvent & e)
 {
 #ifdef __WINDOWS__
@@ -1609,6 +1683,45 @@ void MainFrame::OnRemoveCPinMasks(wxCommandEvent & e)
     };
 }
 
+#ifdef HUGIN_HSI
+void MainFrame::OnPythonScript(wxCommandEvent & e)
+{
+    wxString fname;
+    wxFileDialog dlg(this,
+		     _("Select python script"),
+		     wxConfigBase::Get()->Read(wxT("/lensPath"),wxT("")), wxT(""),
+		     _("Python script (*.py)|*.py|All files (*.*)|*.*"),
+		     wxFD_OPEN, wxDefaultPosition);
+    dlg.SetDirectory(wxConfigBase::Get()->Read(wxT("/pythonScriptPath"),wxT("")));
+
+    if (dlg.ShowModal() == wxID_OK) {
+        wxString filename = dlg.GetPath();
+        wxConfig::Get()->Write(wxT("/pythonScriptPath"), dlg.GetDirectory());
+        std::string scriptfile((const char *)filename.mb_str(HUGIN_CONV_FILENAME));
+        GlobalCmdHist::getInstance().addCommand(
+            new PythonScriptPanoCmd(pano,scriptfile)
+            );
+    }
+}
+
+void MainFrame::OnPlugin(wxCommandEvent & e)
+{
+    wxFileName file=m_plugins[e.GetId()];
+    if(file.FileExists())
+    {
+        std::string scriptfile((const char *)file.GetFullPath().mb_str(HUGIN_CONV_FILENAME));
+        GlobalCmdHist::getInstance().addCommand(
+                                 new PythonScriptPanoCmd(pano,scriptfile)
+                                 );
+    }
+    else
+    {
+        wxMessageBox(wxString::Format(wxT("Python-Script %s not found.\nStopping processing."),file.GetFullPath().c_str()),_("Warning"),wxOK|wxICON_INFORMATION);
+    };
+}
+
+#endif
+
 void MainFrame::OnUndo(wxCommandEvent & e)
 {
     DEBUG_TRACE("OnUndo");
@@ -1621,7 +1734,6 @@ void MainFrame::OnRedo(wxCommandEvent & e)
     GlobalCmdHist::getInstance().redo();
 }
 
-
 void MainFrame::ShowCtrlPoint(unsigned int cpNr)
 {
     DEBUG_DEBUG("Showing control point " << cpNr);
@@ -1743,6 +1855,11 @@ const wxString & MainFrame::GetXRCPath()
      return huginApp::Get()->GetXRCPath();
 };
 
+const wxString & MainFrame::GetDataPath()
+{
+    return wxGetApp().GetDataPath();
+};
+
 /// hack.. kind of a pseudo singleton...
 MainFrame * MainFrame::Get()
 {
@@ -1886,4 +2003,3 @@ GLPreviewFrame * MainFrame::getGLPreview()
 }
 
 MainFrame * MainFrame::m_this = 0;
-
