@@ -80,6 +80,7 @@ BEGIN_EVENT_TABLE(PreferencesDialog, wxDialog)
     EVT_BUTTON(XRCID("pref_cpdetector_save"), PreferencesDialog::OnCPDetectorSave)
     EVT_BUTTON(XRCID("pref_cpdetector_help"), PreferencesDialog::OnCPDetectorHelp)
     EVT_CHOICE(XRCID("pref_ldr_output_file_format"), PreferencesDialog::OnFileFormatChanged)
+    EVT_CHOICE(XRCID("pref_processor_gui"), PreferencesDialog::OnProcessorChanged)
 //  EVT_CLOSE(RunOptimizerFrame::OnClose)
 END_EVENT_TABLE()
 
@@ -407,7 +408,17 @@ void PreferencesDialog::UpdateDisplayData(int panel)
 
     if (panel==0 || panel == 1) {
         // memory setting
-        unsigned long mem = cfg->Read(wxT("/ImageCache/UpperBound"), HUGIN_IMGCACHE_UPPERBOUND);
+        unsigned long long mem = cfg->Read(wxT("/ImageCache/UpperBound"), HUGIN_IMGCACHE_UPPERBOUND);
+#ifdef __WXMSW__
+        unsigned long mem_low = cfg->Read(wxT("/ImageCache/UpperBound"), HUGIN_IMGCACHE_UPPERBOUND);
+        unsigned long mem_high = cfg->Read(wxT("/ImageCache/UpperBoundHigh"), (long) 0);
+        if (mem_high > 0) {
+          mem = ((unsigned long long) mem_high << 32) + mem_low;
+        }
+        else {
+          mem = mem_low;
+        }
+#endif
         MY_SPIN_VAL("prefs_cache_UpperBound", mem >> 20);
 
         // number of threads
@@ -456,9 +467,6 @@ void PreferencesDialog::UpdateDisplayData(int panel)
         // tempdir
         MY_STR_VAL("prefs_misc_tempdir", cfg->Read(wxT("tempDir"),wxT("")));
 
-        // show druid
-        MY_BOOL_VAL("prefs_misc_showDruid", cfg->Read(wxT("/PreviewFrame/showDruid"),HUGIN_PREVIEW_SHOW_DRUID) != 0l);
-
     }
 
     if (panel==0 || panel == 2) {
@@ -542,6 +550,22 @@ void PreferencesDialog::UpdateDisplayData(int panel)
         MY_SPIN_VAL("pref_jpeg_quality", cfg->Read(wxT("/output/jpeg_quality"), HUGIN_JPEG_QUALITY));
         UpdateFileFormatControls();
 
+        /////
+        /// PROCESSOR
+        MY_CHOICE_VAL("pref_processor_gui", cfg->Read(wxT("/Processor/gui"), HUGIN_PROCESSOR_GUI));
+        t = cfg->Read(wxT("/Processor/start"), HUGIN_PROCESSOR_START) == 1;
+        MY_BOOL_VAL("pref_processor_start", t);
+        t = cfg->Read(wxT("/Processor/parallel"), HUGIN_PROCESSOR_PARALLEL) == 1;
+        MY_BOOL_VAL("pref_processor_parallel", t);
+        t = cfg->Read(wxT("/Processor/overwrite"), HUGIN_PROCESSOR_OVERWRITE) == 1;
+        MY_BOOL_VAL("pref_processor_overwrite", t);
+        t = cfg->Read(wxT("/Processor/verbose"), HUGIN_PROCESSOR_VERBOSE) == 1;
+        MY_BOOL_VAL("pref_processor_verbose", t);
+        UpdateProcessorControls();
+    }
+
+    if (panel==0 || panel == 6){
+
         /////
         /// NONA
         MY_CHOICE_VAL("prefs_nona_interpolator", cfg->Read(wxT("/Nona/Interpolator"), HUGIN_NONA_INTERPOLATOR));
@@ -574,7 +598,7 @@ void PreferencesDialog::UpdateDisplayData(int panel)
                                                       wxT(HUGIN_ENFUSE_ARGS)));
     }
     
-    if (panel==0 || panel == 6) {
+    if (panel==0 || panel == 7) {
         // Celeste settings
 
         d=HUGIN_CELESTE_THRESHOLD;
@@ -626,6 +650,14 @@ void PreferencesDialog::OnRestoreDefaults(wxCommandEvent & e)
         if (noteb->GetSelection() == 0) {
             // MISC
             // cache
+/*
+ * special treatment for windows not necessary here since we know the value of
+ * HUGIN_IMGCACHE_UPPERBOUND must fit into 32bit to be compatible with 32bit systems.
+ * However, just as a reminder:
+#ifdef __WXMSW__
+    cfg->Write(wxT("/ImageCache/UpperBoundHigh"), HUGIN_IMGCACHE_UPPERBOUND >> 32);
+#endif
+*/
             cfg->Write(wxT("/ImageCache/UpperBound"), HUGIN_IMGCACHE_UPPERBOUND);
             // number of threads
             int cpucount = wxThread::GetCPUCount();
@@ -639,8 +671,6 @@ void PreferencesDialog::OnRestoreDefaults(wxCommandEvent & e)
             cfg->Write(wxT("smartUndo"), HUGIN_SMART_UNDO);
             // projection hints
             cfg->Write(wxT("/GLPreviewFrame/ShowProjectionHints"), HUGIN_SHOW_PROJECTION_HINTS);
-            // druid
-            cfg->Write(wxT("/PreviewFrame/showDruid"), HUGIN_PREVIEW_SHOW_DRUID);
         }
         if (noteb->GetSelection() == 1) {
             cfg->Write(wxT("/Assistant/autoAlign"), HUGIN_ASS_AUTO_ALIGN);
@@ -684,6 +714,15 @@ void PreferencesDialog::OnRestoreDefaults(wxCommandEvent & e)
             // cfg->Write(wxT("/output/hdr_format"), HUGIN_HDR_OUTPUT_FORMAT);
             cfg->Write(wxT("/output/tiff_compression"), HUGIN_TIFF_COMPRESSION);
             cfg->Write(wxT("/output/jpeg_quality"), HUGIN_JPEG_QUALITY);
+            // stitching engine
+            cfg->Write(wxT("/Processor/gui"), HUGIN_PROCESSOR_GUI);
+            cfg->Write(wxT("/Processor/start"), HUGIN_PROCESSOR_START);
+            cfg->Write(wxT("/Processor/parallel"), HUGIN_PROCESSOR_PARALLEL);
+            cfg->Write(wxT("/Processor/overwrite"), HUGIN_PROCESSOR_OVERWRITE);
+            cfg->Write(wxT("/Processor/verbose"), HUGIN_PROCESSOR_VERBOSE);
+
+        }
+        if (noteb->GetSelection() == 5) {
 
             /// ENBLEND
             cfg->Write(wxT("/Enblend/Exe"), wxT(HUGIN_ENBLEND_EXE));
@@ -695,7 +734,7 @@ void PreferencesDialog::OnRestoreDefaults(wxCommandEvent & e)
             cfg->Write(wxT("/Enfuse/Args"), wxT(HUGIN_ENFUSE_ARGS));
         }
 	
-        if (noteb->GetSelection() == 5) {
+        if (noteb->GetSelection() == 6) {
             /// Celeste
             cfg->Write(wxT("/Celeste/Threshold"), HUGIN_CELESTE_THRESHOLD);
             cfg->Write(wxT("/Celeste/Filter"), HUGIN_CELESTE_FILTER);
@@ -761,6 +800,10 @@ void PreferencesDialog::UpdateConfigData()
     /////
     /// MISC
     // cache
+#ifdef __WXMSW__
+    // shifting only 12 bits rights: 32-20=12 and the prefs_cache_UpperBound is in GB
+    cfg->Write(wxT("/ImageCache/UpperBoundHigh"), (long) MY_G_SPIN_VAL("prefs_cache_UpperBound") >> 12);
+#endif
     cfg->Write(wxT("/ImageCache/UpperBound"), (long) MY_G_SPIN_VAL("prefs_cache_UpperBound") << 20);
     // number of threads
     cfg->Write(wxT("/Nona/NumberOfThreads"), MY_G_SPIN_VAL("prefs_nona_NumberOfThreads"));
@@ -786,9 +829,6 @@ void PreferencesDialog::UpdateConfigData()
     //    cfg->Write(wxT("/CPImageCtrl/CursorType"), MY_G_SPIN_VAL("prefs_cp_CursorType"));
     // tempdir
     cfg->Write(wxT("tempDir"),MY_G_STR_VAL("prefs_misc_tempdir"));
-    // druid
-    cfg->Write(wxT("/PreviewFrame/showDruid"), MY_G_BOOL_VAL("prefs_misc_showDruid"));
-    
     /////
     /// AUTOPANO
     cpdetector_config_edit.Write(cfg);
@@ -801,6 +841,14 @@ void PreferencesDialog::UpdateConfigData()
     cfg->Write(wxT("/output/tiff_compression"), MY_G_CHOICE_VAL("pref_tiff_compression"));
     cfg->Write(wxT("/output/jpeg_quality"), MY_G_SPIN_VAL("pref_jpeg_quality"));
 
+    /////
+    /// PROCESSOR
+    cfg->Write(wxT("/Processor/gui"), MY_G_CHOICE_VAL("pref_processor_gui"));
+    cfg->Write(wxT("/Processor/start"), MY_G_BOOL_VAL("pref_processor_start"));
+    cfg->Write(wxT("/Processor/parallel"), MY_G_BOOL_VAL("pref_processor_parallel"));
+    cfg->Write(wxT("/Processor/overwrite"), MY_G_BOOL_VAL("pref_processor_overwrite"));
+    cfg->Write(wxT("/Processor/verbose"), MY_G_BOOL_VAL("pref_processor_verbose"));
+
     /////
     /// STITCHING
     cfg->Write(wxT("/Nona/Interpolator"), MY_G_CHOICE_VAL("prefs_nona_interpolator"));
@@ -975,4 +1023,35 @@ void PreferencesDialog::UpdateFileFormatControls()
     XRCCTRL(*this,"pref_jpeg_quality_label",wxStaticText)->Show(i==1);
     XRCCTRL(*this,"pref_jpeg_quality",wxSpinCtrl)->Show(i==1);
     XRCCTRL(*this,"pref_tiff_compression",wxChoice)->GetParent()->Layout();
-};
\ No newline at end of file
+};
+
+void PreferencesDialog::OnProcessorChanged(wxCommandEvent &e)
+{
+    UpdateProcessorControls();
+};
+
+void PreferencesDialog::UpdateProcessorControls()
+{
+    int i=MY_G_CHOICE_VAL("pref_processor_gui");
+    XRCCTRL(*this,"pref_processor_start",wxCheckBox)->Enable(i==0);
+    XRCCTRL(*this,"pref_processor_parallel",wxCheckBox)->Enable(i==0);
+    XRCCTRL(*this,"pref_processor_verbose",wxCheckBox)->Enable(i==0);
+    switch(i)
+    {
+        case 0:
+            //PTBatcherGUI
+            {
+                wxConfigBase* config=wxConfigBase::Get();
+                XRCCTRL(*this,"pref_processor_start",wxCheckBox)->SetValue(config->Read(wxT("/Processor/start"), HUGIN_PROCESSOR_START) == 1);
+                XRCCTRL(*this,"pref_processor_parallel",wxCheckBox)->SetValue(config->Read(wxT("/Processor/parallel"), HUGIN_PROCESSOR_PARALLEL) == 1);
+                XRCCTRL(*this,"pref_processor_verbose",wxCheckBox)->SetValue(config->Read(wxT("/Processor/verbose"), HUGIN_PROCESSOR_VERBOSE) == 1);
+            }
+            break;
+        case 1:
+            //Hugin_stitch_project
+            XRCCTRL(*this,"pref_processor_start",wxCheckBox)->SetValue(true);
+            XRCCTRL(*this,"pref_processor_parallel",wxCheckBox)->SetValue(false);
+            XRCCTRL(*this,"pref_processor_verbose",wxCheckBox)->SetValue(true);
+            break;
+    };
+};
