@@ -38,7 +38,11 @@
 #include <vigra/localminmax.hxx>
 #include <panodata/StandardImageVariableGroups.h>
 
-#include <hugin_utils/alphanum.hpp>
+#include <hugin_utils/alphanum.h>
+
+#ifdef HUGIN_HSI
+#include "hugin_script_interface/hpi.h"
+#endif
 
 using namespace std;
 using namespace vigra;
@@ -150,7 +154,7 @@ void wxAddImagesCmd::execute()
     switch (sort) {
         case 1:
                 // sort by filename
-            std::sort(files.begin(), files.end(), doj::alphanum_less<std::string>());
+            std::sort(files.begin(), files.end(), doj::alphanum_less());
             break;
         case 2:
                 // sort by date
@@ -168,7 +172,6 @@ void wxAddImagesCmd::execute()
     double focalLength = 0;
 
     Lens lens;
-    ImageOptions imgopts;
     SrcPanoImage srcImg;
     HuginBase::StandardImageVariableGroups variable_groups(pano);
     HuginBase::ImageVariableGroup & lenses = variable_groups.getLenses();
@@ -226,6 +229,7 @@ void wxAddImagesCmd::execute()
                         double ev = srcImg.getExposureValue();
                         srcImg = pano.getSrcImage(i);
                         srcImg.setFilename(filename);
+                        srcImg.deleteAllMasks();
                         // add image
                         int imgNr = pano.addImage(srcImg);
                         variable_groups.update();
@@ -284,6 +288,7 @@ void wxAddImagesCmd::execute()
                     set_exposure = true;
                     srcImg = pano.getSrcImage(i);
                     srcImg.setFilename(filename);
+                    srcImg.deleteAllMasks();
                     srcImg.setExposureValue(ev);
                     break;
                 }
@@ -295,6 +300,7 @@ void wxAddImagesCmd::execute()
                     // the file name
                     srcImg = pano.getSrcImage(i);
                     srcImg.setFilename(filename);
+                    srcImg.deleteAllMasks();
                     break;
                 }
             }
@@ -691,5 +697,30 @@ void wxApplyTemplateCmd::execute()
     pano.changeFinished();
 }
 
+#ifdef HUGIN_HSI
+bool PythonScriptPanoCmd::processPanorama(Panorama& pano)
+{
+    std::cout << "run python script: " << m_scriptFile.c_str() << std::endl;
+
+    int success = hpi::callhpi ( m_scriptFile.c_str() , 1 ,
+                   "HuginBase::Panorama*" , &pano ) ;
+
+    if(success!=0)
+        wxMessageBox(wxString::Format(wxT("Script returned %d"),success),_("Result"), wxICON_INFORMATION);
+    std::cout << "Python interface returned " << success << endl ;
+    // notify other of change in panorama
+    if(pano.getNrOfImages()>0)
+    {
+        for(unsigned int i=0;i<pano.getNrOfImages();i++)
+        {
+            pano.imageChanged(i);
+        };
+    };
+    pano.changeFinished();
+
+    return true;
+}
+#endif
+
 } // namespace
 
