@@ -159,7 +159,12 @@ bool SrcPanoImage::isInside(vigra::Point2D p, bool ignoreMasks) const
         return insideCrop;
 }
 
-    
+bool SrcPanoImage::isCircularCrop()
+{
+    HuginBase::BaseSrcPanoImage::Projection projection=m_Projection.getData();
+    return (projection==CIRCULAR_FISHEYE || projection==FISHEYE_THOBY);
+};
+
 bool SrcPanoImage::getCorrectTCA() const
 { 
     bool nr = (m_RadialDistortionRed.getData()[0] == 0.0 && m_RadialDistortionRed.getData()[1] == 0.0 &&
@@ -261,52 +266,6 @@ VariableMap SrcPanoImage::getVariableMap() const
     return vars;
 }
 
-
-ImageOptions SrcPanoImage::getOptions() const
-{
-    // convert stuff to ImageOptions object for old stuff not using the new interface.
-    ImageOptions opts;
-    opts.featherWidth = m_FeatherWidth.getData();
-    opts.morph = m_Morph.getData();
-    opts.docrop = m_CropMode.getData() != NO_CROP;
-    opts.autoCenterCrop = m_AutoCenterCrop.getData();
-    opts.cropRect = m_CropRect.getData();
-    opts.m_vigCorrMode = m_VigCorrMode.getData();
-    opts.m_flatfield = m_FlatfieldFilename.getData();
-    opts.responseType = m_ResponseType.getData();
-    opts.active = m_Active.getData();
-    
-    return opts;
-}
-
-void SrcPanoImage::setOptions(const ImageOptions & opts)
-{
-    // set variables from an ImageOptions object for the stuff using the old interface.
-    m_FeatherWidth.setData(opts.featherWidth);
-    m_Morph.setData(opts.morph);
-    if (opts.docrop)
-    {
-        if (m_Projection.getData() == CIRCULAR_FISHEYE || m_Projection.getData() == FISHEYE_THOBY)
-        {
-            m_CropMode.setData(CROP_CIRCLE);
-        }
-        else
-        {
-            m_CropMode.setData(CROP_RECTANGLE);
-        }
-        m_CropRect.setData(opts.cropRect);
-    } else {
-        m_CropMode.setData(NO_CROP);
-        m_CropRect.setData((vigra::Rect2D)m_Size.getData());
-    }
-    m_AutoCenterCrop.setData(opts.autoCenterCrop);    
-    m_VigCorrMode.setData(opts.m_vigCorrMode);
-    m_FlatfieldFilename.setData(opts.m_flatfield);
-    m_ResponseType.setData((ResponseType)opts.responseType);
-    m_Active.setData(opts.active);
-}
-
-
 bool SrcPanoImage::readEXIF(double & focalLength, double & cropFactor, bool applyEXIFValues, bool applyExposureValue)
 {
     double eV=0;
@@ -369,6 +328,12 @@ bool SrcPanoImage::readEXIF(double & focalLength, double & cropFactor, double &
     
     //remember aperture for later
     setExifAperture(photoFNumber);
+    
+    //read exposure mode
+    long exposureMode=0;
+    getExiv2Value(exifData,"Exif.Photo.ExposureMode",exposureMode);
+    setExifExposureMode((int)exposureMode);
+
     //if no F-number was found in EXIF data assume a f stop of 3.5 to get
     //a reasonable ev value if shutter time, e. g. for manual lenses is found
     if(photoFNumber==0)
@@ -879,6 +844,12 @@ void SrcPanoImage::deleteMask(unsigned int index)
     };
 };
 
+void SrcPanoImage::deleteAllMasks()
+{
+    MaskPolygonVector emptyMaskVector;
+    m_Masks.setData(emptyMaskVector);
+};
+
 bool SrcPanoImage::isInsideMasks(vigra::Point2D p) const
 {
     if(!hasActiveMasks())
@@ -907,4 +878,13 @@ bool SrcPanoImage::trustExivOrientation()
     return true;
 }
 
+const int SrcPanoImage::getExifDateTime(struct tm* datetime) const
+{
+    //initialize struct
+    std::memset(datetime, 0x0, sizeof(*datetime));
+    //ignore daylight saving flag because it is not saved in EXIF date time format
+    datetime->tm_isdst=-1;
+    return Exiv2::exifTime(m_ExifDate.getData().c_str(),datetime);
+};
+
 } // namespace
